'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var serializeQueryParams = require('serialize-query-params');
var reactRouterDom = require('react-router-dom');
var react = require('react');

/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license at
 * https://github.com/facebook/fbjs/blob/master/LICENSE
 */

/*eslint-disable no-self-compare */
var hasOwnProperty = Object.prototype.hasOwnProperty;
/**
 * inlined Object.is polyfill to avoid requiring consumers ship their own
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/is
 */

function is(x, y) {
  // SameValue algorithm
  if (x === y) {
    // Steps 1-5, 7-10
    // Steps 6.b-6.e: +0 != -0
    // Added the nonzero y check to make Flow happy, but it is redundant
    return x !== 0 || y !== 0 || 1 / x === 1 / y;
  } else {
    // Step 6.a: NaN == NaN
    return x !== x && y !== y;
  }
}
/**
 * Performs equality by iterating through keys on an object and returning false
 * when any key has values which are not strictly equal between the arguments.
 * Returns true when the values of all keys are strictly equal.

 * @pbeshai modification of shallowEqual to take into consideration a map providing
 * equals functions
 */


function shallowEqual(objA, objB, equalMap) {
  if (is(objA, objB)) {
    return true;
  }

  if (typeof objA !== 'object' || objA === null || typeof objB !== 'object' || objB === null) {
    return false;
  }

  var keysA = Object.keys(objA);
  var keysB = Object.keys(objB);

  if (keysA.length !== keysB.length) {
    return false;
  } // Test for A's keys different from B.


  for (var i = 0; i < keysA.length; i++) {
    var _equalMap$keysA$i$equ, _equalMap$keysA$i;

    var isEqual = (_equalMap$keysA$i$equ = equalMap == null ? void 0 : (_equalMap$keysA$i = equalMap[keysA[i]]) == null ? void 0 : _equalMap$keysA$i.equals) != null ? _equalMap$keysA$i$equ : is;

    if (!hasOwnProperty.call(objB, keysA[i]) || !isEqual(objA[keysA[i]], objB[keysA[i]])) {
      return false;
    }
  }

  return true;
}

/**
 * Given a query parameter configuration (mapping query param name to { encode, decode }),
 * return an object with the decoded values and a setter for updating them.
 */

var useQueryParams = function useQueryParams(paramConfigMap) {
  var paramConfigMapRef = react.useRef(paramConfigMap);
  paramConfigMap = shallowEqual(paramConfigMap, paramConfigMapRef.current) ? paramConfigMapRef.current : paramConfigMap;
  paramConfigMapRef.current = paramConfigMap;

  var _useSearchParams = reactRouterDom.useSearchParams(),
      searchParams = _useSearchParams[0],
      setSearchParams = _useSearchParams[1];

  var searchParamsStringified = searchParams.toString();
  var searchParamsRef = react.useRef(searchParams);
  searchParamsRef.current = searchParams;
  var result = react.useMemo(function () {
    var value = {};
    Object.keys(paramConfigMap).forEach(function (key) {
      value[key] = searchParams.getAll(key);
    });
    return serializeQueryParams.decodeQueryParams(paramConfigMap, value);
  }, [searchParamsStringified]);
  var setValue = react.useCallback(function (value, navigateOptions) {
    var encoded = serializeQueryParams.encodeQueryParams(paramConfigMap, value);
    Object.keys(value).forEach(function (key) {
      var keyValue = encoded[key];

      if (Array.isArray(keyValue)) {
        keyValue.forEach(function (v, index) {
          if (index === 0) searchParamsRef.current.set(key, v);else searchParamsRef.current.append(key, v);
        });
      } else {
        searchParamsRef.current.set(key, keyValue);
      }
    });
    setSearchParams(searchParamsRef.current, navigateOptions);
  }, [paramConfigMap, setSearchParams]);
  return react.useMemo(function () {
    return [result, setValue];
  }, [searchParamsStringified, setValue]);
};
/**
 * Helper to get the latest decoded value with smart caching.
 * Abstracted into its own function to allow re-use in a functional setter (#26)
 */

var useQueryParam = function useQueryParam(name, paramConfig) {
  var _useQueryParams2;

  if (paramConfig === void 0) {
    paramConfig = serializeQueryParams.StringParam;
  }

  var _useQueryParams = useQueryParams((_useQueryParams2 = {}, _useQueryParams2[name] = paramConfig, _useQueryParams2)),
      queryParams = _useQueryParams[0],
      setQueryParams = _useQueryParams[1];

  var setValue = react.useCallback(function (value) {
    var _setQueryParams;

    setQueryParams((_setQueryParams = {}, _setQueryParams[name] = value, _setQueryParams));
  }, []);
  return react.useMemo(function () {
    return [queryParams[name], setValue];
  }, [queryParams[name], setValue]);
};

function createRoute(pattern, paramTypes) {
  return {
    route: pattern == null ? void 0 : pattern.toString(),
    pattern: pattern,
    link: function link(params, search) {
      var encodedParams = paramTypes ? serializeQueryParams.encodeQueryParams(paramTypes, params) : params;
      var result = reactRouterDom.generatePath(pattern, encodedParams);

      if (search) {
        result = result + '?' + reactRouterDom.createSearchParams(search);
        console.log('qwe', result);
      }

      return result.replace('*', '');
    },
    useParams: function useParams() {
      var params = reactRouterDom.useParams();

      if (paramTypes) {
        return serializeQueryParams.decodeQueryParams(paramTypes, params);
      }

      return params;
    },
    useMatch: function useMatch() {
      var match = reactRouterDom.useMatch(pattern);

      if (match) {
        if (paramTypes) match.params = serializeQueryParams.decodeQueryParams(paramTypes, match.params);
      }

      return match;
    },
    matchPath: function matchPath(path) {
      var match = reactRouterDom.matchPath(pattern, path);

      if (match) {
        if (paramTypes) match.params = serializeQueryParams.decodeQueryParams(paramTypes, match.params);
      }

      return match;
    }
  };
}

var RequiredNumberParam = {
  encode: function encode(value) {
    return serializeQueryParams.NumberParam.encode(value);
  },
  decode: function decode(value) {
    var result = serializeQueryParams.NumberParam.decode(value);

    if (result === null || result === undefined || isNaN(result)) {
      throw new Error("Unable to convert '" + value + "' to number");
    }

    return result;
  }
};
var RequiredStringParam = {
  encode: function encode(value) {
    return serializeQueryParams.StringParam.encode(value);
  },
  decode: function decode(value) {
    var result = serializeQueryParams.StringParam.decode(value);

    if (result === null || result === undefined) {
      throw new Error("Param isnt defined");
    }

    return result;
  }
};

exports.RequiredNumberParam = RequiredNumberParam;
exports.RequiredStringParam = RequiredStringParam;
exports.createRoute = createRoute;
exports.useQueryParam = useQueryParam;
exports.useQueryParams = useQueryParams;
//# sourceMappingURL=react-router-url-params.cjs.development.js.map
